#ifndef LOWLEVEL_H
#define LOWLEVEL_H
/*
* Dateiname.... lowlevel.h
* erstellt am.. 2013-5-25
* gendert am.. 2014-03-02
* Autor........ Clemens B Fahr
* email........ clemens.fahr[at]fahrnet.de
* Kommentar.... based on RFM_wireless_small_base_01 SVN 579
* .............	switch to ISR approach for RFM-RX
* .............	
* ............. 
*/

// includes
//------------------------------------------------------------------
#include <avr\io.h>
#include <inttypes.h>

// defines
//------------------------------------------------------------------
// clock prescaler: external clock 8MHz => 2MHz f_CPU => divider 4
#define CLK_PRESCALER (0x01 << CLKPS1)

// timer0 clock source for the UART bit delay routine
// at 2MHz clock use divider 1 yielding 2MHz
#define CLK_SRC_BIT_T0 (0x01 << CS00)
// timer0 counter preload value, count up from 201 to 256 gives overflow after 52.1us (19200 baud) => measured with AVR studio for correct baud-timing
#define PRELOAD_BIT_T0 200
#define TIMER0_CLK_OFF ~(0x01 << CS02) & ~(0x01 << CS01) & ~(0x01 << CS00)

// timer1 clock source for the 1ms-delay-routine
// at 2MHz clock use divider 16 yielding 125kHz
#define CLK_SRC_1MS_T1 ( (0x01 << CS12) | (0x01 << CS10) )
// timer1 counter preload value, count up from 131 to 256 gives overflow after 1ms
#define PRELOAD_1MS_T1	131
// timer1 clock source for the 200ms-delay-routine
// at 2MHz clock use divider 4096 yielding 490Hz
#define CLK_SRC_500MS_T1 ((0x01 << CS13) | (0x01 << CS12) | (0x01 << CS10) )
// timer1 counter preload value, count up from 158 to 256 gives overflow after 200ms; resp 207 to 256 of 100ms
#define PRELOAD_500MS_T1	158
#define TIMER1_CLK_OFF ~(0x01 << CS13) & ~(0x01 << CS12) & ~(0x01 << CS11) & ~(0x01 << CS10)

// variables
//------------------------------------------------------------------
extern volatile uint8_t telegramComplete;
extern volatile uint8_t numberReceivedBytes;
extern volatile uint8_t RX_buffer[];
extern volatile uint8_t LED1_status;
extern volatile uint8_t LED1_blink_count;
extern volatile uint8_t LED1_enabled;

// forward and external declarations
//------------------------------------------------------------------
void LOWLEVEL_init_all(void);
void LOWLEVEL_init_RFM(void);
void LOWLEVEL_RFM_stop_RX(void);
void LOWLEVEL_delay_1ms(uint16_t multiplier);
void LOWLEVEL_UART_TX(uint8_t aByte);
void LOWLEVEL_toggle_LED1(uint8_t inhibit);

uint16_t send_command_RFM(uint16_t aCommand);
void UART_bit_delay(void);
void run_timer0(uint8_t preloadvalue, uint8_t clocksource, uint8_t IRQ_enabled);
void stop_timer0(void);
void run_timer1(uint8_t preloadvalue, uint8_t clocksource, uint8_t IRQ_enabled);
void stop_timer1(void);
void toggle_LED1(void);

#endif